/*
 *  Helper functions for outputting text.
 *  Copyright © 2011, 2021, 2023 Nick Bowler
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <config.h>
#include <stdio.h>
#include <assert.h>

#include "cdecl.h"
#include "cdecl-internal.h"

#include "parse.h"
#include "specstr.h"

#define MIN(a, b) ((a) < (b) ? (a) : (b))

size_t cdecl__advance(struct output_state *dst, size_t amount)
{
	size_t x = MIN(amount, dst->dstlen);

	dst->dst    += x;
	dst->dstlen -= x;
	dst->accum  += amount;

	return amount;
}

size_t cdecl__emit(struct output_state *dst, const char *src)
{
	size_t rc = snprintf(dst->dst, dst->dstlen, "%s", src);
	return cdecl__advance(dst, rc);
}

static void explain_spec(struct output_state *dst, struct cdecl_declspec *s)
{
	size_t rc;

	rc = cdecl__emit(dst, spec_string(s->type));
	if (s->ident) {
		cdecl__emit(dst, " " + !rc);
		cdecl__emit(dst, s->ident);
	}
}

/*
 * Render a list of declaration specifiers.  Only the declaration specifiers
 * listed in mask, which is the bitwise OR of the desired specifier kinds, are
 * printed.
 */
const char *cdecl__emit_specs(struct output_state *dst,
                              struct cdecl_declspec *s,
                              unsigned mask)
{
	const char *sep = " ";
	int empty = 1;

	for (; s; s = s->next) {
		if (!(s->type & mask))
			continue;

		cdecl__emit(dst, sep + empty);
		explain_spec(dst, s);
		empty = 0;
	}

	return sep + empty;
}
